use std::rc::Rc;

use crate::font::{fontdb as fdb, FontStore};

use kahlo::colour::Colour;

pub struct Theme {
    pub active: Colour,
    pub border: Colour,
    pub border_width: usize,
    pub background: Colour,
    pub panel: Colour,
    pub foreground: Colour,

    pub ui_font: Rc<fontdue::Font>,
    pub ui_font_size: f32,
}

impl Theme {
    pub fn build_default(fs: &FontStore) -> Self {
        let ui_font = fs
            .query_for_font(&fdb::Query {
                families: &[fdb::Family::SansSerif],
                ..Default::default()
            })
            .expect("couldn't load sans-serif font!");

        Self {
            active: Colour::hex_rgb("#c16e70").unwrap(),
            border: Colour::hex_rgb("#dc9e82").unwrap(),
            border_width: 3,
            background: Colour::hex_rgb("#030027").unwrap(),
            panel: Colour::hex_rgb("#151e3f").unwrap(),
            foreground: Colour::WHITE,

            // other colour in palette: #F2F3D9
            ui_font,
            ui_font_size: 16.0,
        }
    }
}
